package com.yc.ship.module.ota.admin.tradelog.vo;

import com.yc.ship.framework.common.pojo.PageParam;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.time.LocalDateTime;

import static com.yc.ship.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

@Schema(description = "管理后台 - 分销商交易日志分页 Request VO")
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
public class TradeLogPageReqVO extends PageParam {

    @Schema(description = "操作人ID", example = "30405")
    private Long operatorId;

    @Schema(description = "操作人名字", example = "芋艿")
    private String operatorName;

    @Schema(description = "1 余额充值、2 预付购票、3 预付退款、4 余额购票、5余额退票", example = "2")
    private Integer tradeType;

    @Schema(description = "交易时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDateTime[] tradeTime;

    private String tradeStartTime;

    private String tradeEndTime;

    @Schema(description = "交易前余额")
    private BigDecimal beforeBalance;

    @Schema(description = "交易后余额")
    private BigDecimal afterBalance;

    @Schema(description = "交易金额")
    private BigDecimal tradeAmount;

    @Schema(description = "1  增加，2 减少")
    private Integer tradeMode;

    @Schema(description = "备注", example = "你说的对")
    private String remark;

    @Schema(description = "订单号", example = "18147")
    private String orderNo;

    @Schema(description = "分销商id", example = "22794")
    private Long distributorId;

    @Schema(description = "创建时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private LocalDateTime[] createTime;

    @Schema(description = "门店id", example = "22794")
    private Long storeId;

}
