package com.yc.ship.module.ota.service.distributorrecharge;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.yc.ship.framework.common.exception.ServiceException;
import com.yc.ship.framework.common.lang.Asserts;
import com.yc.ship.framework.common.pojo.CommonResult;
import com.yc.ship.framework.common.pojo.PageResult;
import com.yc.ship.framework.common.util.object.BeanUtils;
import com.yc.ship.framework.mybatis.core.util.MyBatisUtils;
import com.yc.ship.framework.tenant.core.aop.TenantIgnore;
import com.yc.ship.module.ota.admin.distributorrecharge.vo.DistributorRechargeAuditReqVO;
import com.yc.ship.module.ota.admin.distributorrecharge.vo.DistributorRechargePageReqVO;
import com.yc.ship.module.ota.admin.distributorrecharge.vo.DistributorRechargeRespVO;
import com.yc.ship.module.ota.admin.distributorrecharge.vo.DistributorRechargeSaveReqVO;
import com.yc.ship.module.ota.admin.tradelog.vo.TradeLogSaveReqVO;
import com.yc.ship.module.ota.api.dto.DistributorRechargeReqDTO;
import com.yc.ship.module.ota.api.dto.DistributorRechargeRespDTO;
import com.yc.ship.module.ota.dal.dataobject.distributor.DistributorDO;
import com.yc.ship.module.ota.dal.dataobject.distributorrecharge.DistributorRechargeDO;
import com.yc.ship.module.ota.dal.dataobject.tradelog.TradeLogDO;
import com.yc.ship.module.ota.dal.mysql.distributorrecharge.DistributorRechargeMapper;
import com.yc.ship.module.ota.enums.AuditStatusEnum;
import com.yc.ship.module.ota.enums.DistributorTradeTypeEnum;
import com.yc.ship.module.ota.enums.DistributorTypeEnum;
import com.yc.ship.module.ota.service.distributor.DistributorService;
import com.yc.ship.module.ota.service.tradelog.TradeLogService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;
import java.util.function.Supplier;
import java.util.stream.Stream;

import static com.yc.ship.framework.common.exception.util.ServiceExceptionUtil.exception;
import static com.yc.ship.framework.common.exception.util.ServiceExceptionUtil.exception0;
import static com.yc.ship.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;
import static com.yc.ship.module.ota.enums.ErrorCodeConstants.*;

/**
 * 分销商充值 Service 实现类
 *
 * @author 管理员
 */
@Service
@Validated
@Slf4j
public class DistributorRechargeServiceImpl implements DistributorRechargeService {


    @Resource
    private DistributorRechargeMapper distributorRechargeMapper;

    @Resource
    private DistributorService distributorService;

    @Resource
    private RedissonClient redissonClient;
    @Resource
    private TradeLogService tradeLogService;


    /**
     * 余额类型
     */
    private static final int RECHARGE_TYPE_BALANCE = 1;

    /**
     * 授信类型
     */
    private static final int RECHARGE_TYPE_CREDIT = 2;

    /**
     * 定向库存类型
     */
    private static final int RECHARGE_TYPE_INVENTORY = 3;

    @Override
    public Long createDistributorRecharge(DistributorRechargeSaveReqVO createReqVO) {
        // 插入
        DistributorRechargeDO distributorRecharge = BeanUtils.toBean(createReqVO, DistributorRechargeDO.class);
        Long id = IdWorker.getId(distributorRecharge);
        distributorRecharge.setId(id);
        distributorRecharge.setDeleted(false);
        //转账充值
        distributorRecharge.setRechargeMode(1);
        distributorRecharge.setStatus(AuditStatusEnum.PENDING.getValue());
        Long userId = getLoginUserId();
        distributorRecharge.setApplyUser(userId);
        distributorRecharge.setApplyTime(LocalDateTime.now());
        distributorRechargeMapper.insert(distributorRecharge);
        // 返回
        return distributorRecharge.getId();
    }

    @Override
    public void updateDistributorRecharge(DistributorRechargeSaveReqVO updateReqVO) {
        // 校验存在
        validateDistributorRechargeExists(updateReqVO.getId());
        // 更新
        DistributorRechargeDO updateObj = BeanUtils.toBean(updateReqVO, DistributorRechargeDO.class);
        distributorRechargeMapper.updateById(updateObj);
    }

    @Override
    public void deleteDistributorRecharge(Long id) {
        // 校验存在
        validateDistributorRechargeExists(id);
        // 删除
        distributorRechargeMapper.deleteById(id);
    }

    private void validateDistributorRechargeExists(Long id) {
        if (distributorRechargeMapper.selectById(id) == null) {
            throw exception0(500,  "分销商充值不存在");
        }
    }

    @Override
    public DistributorRechargeDO getDistributorRecharge(Long id) {
        return distributorRechargeMapper.selectById(id);
    }

    @Override
    public DistributorRechargeRespVO get(Long id) {
        DistributorRechargeDO distributorRechargeDO = distributorRechargeMapper.selectById(id);
        DistributorRechargeRespVO respVO = BeanUtils.toBean(distributorRechargeDO, DistributorRechargeRespVO.class);
        DistributorDO distributor = distributorService.getDistributor(respVO.getDistributorId());
        respVO.setDistributorName(distributor.getName());
        return respVO;
    }

    @Override
    public PageResult<DistributorRechargeRespVO> getDistributorRechargePage(DistributorRechargePageReqVO pageReqVO) {
        IPage<DistributorRechargeDO> iPage = MyBatisUtils.buildPage(pageReqVO);
        IPage<DistributorRechargeRespVO> page = distributorRechargeMapper.queryPage(iPage, pageReqVO);
        PageResult<DistributorRechargeRespVO> result = new PageResult<>(page.getRecords(), page.getTotal());
        return result;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void auditDistributorRecharge(DistributorRechargeAuditReqVO reqVO) {
        validateDistributorRechargeExists(reqVO.getId());
        DistributorRechargeDO rechargeDO = this.getDistributorRecharge(reqVO.getId());
        if (!ObjectUtil.equal(rechargeDO.getStatus(), AuditStatusEnum.PENDING.getValue())) {
            throw exception(RECHARGE_HAVA_REVIEWED);
        }
        if (reqVO.getAuditStatus().equals(AuditStatusEnum.APPROVED.getValue())) {
            Long logId = this.backDistributorRecharge(rechargeDO);
            rechargeDO.setRechargeLogId(logId);
        }
        rechargeDO.setAuditRemark(reqVO.getAuditRemark());
        rechargeDO.setStatus(reqVO.getAuditStatus());
        rechargeDO.setAuditTime(LocalDateTime.now());
        Long userId = getLoginUserId();
        rechargeDO.setAuditUser(userId);
        distributorRechargeMapper.updateById(rechargeDO);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long backDistributorRecharge(DistributorRechargeDO rechargeDO) {
        if (rechargeDO == null) {
            throw exception(DISTRIBUTOR_RECHARGE_NOT_EXISTS);
        }
        //充值金额
        BigDecimal money = rechargeDO.getMoney();
        String key = "ota_" + rechargeDO.getDistributorId();
        switch (rechargeDO.getRechargeType()) {
            case RECHARGE_TYPE_BALANCE:
                key += "_balance";
                break;
            case RECHARGE_TYPE_CREDIT:
                key += "_credit";
                break;
            case RECHARGE_TYPE_INVENTORY:
                key += "_inventory";
                break;
            default:
                break;
        }
        RLock lock = redissonClient.getLock(key);
        try {
            if (lock.tryLock(10, 30, TimeUnit.SECONDS)) {
                DistributorDO distributorDO = distributorService.getDistributor(rechargeDO.getDistributorId());
                if (distributorDO == null) {
                    throw exception(DISTRIBUTOR_NOT_EXISTS);
                }
                boolean flag = false;
                BigDecimal beforeBalance = BigDecimal.ZERO;
                BigDecimal afterBalance = BigDecimal.ZERO;

                String tradeName = "";
                int tradeType = 1;
                //
                String payType = distributorDO.getPayType();
                if (StringUtils.isBlank(payType)) {

                }
                String[] payTypeArray = payType.split(",");
                Stream<String> typeStream = Arrays.stream(payTypeArray);
                if (ObjectUtil.equals(rechargeDO.getRechargeType(), DistributorTradeTypeEnum.BALANCE_RECHARGE.getType())) {
                    beforeBalance = distributorDO.getBalance();
                    afterBalance = beforeBalance.add(money);
                    tradeName = "余额充值金额：" + money + "元,充值后余额为" + afterBalance + "元;";
                    flag = distributorService.addBalance(distributorDO.getId(), money);
                } else if (ObjectUtil.equals(rechargeDO.getRechargeType(), DistributorTradeTypeEnum.CREDIT_RECHARGE.getType())) {
                    if (typeStream.noneMatch(element -> Integer.parseInt(element) == DistributorTradeTypeEnum.CREDIT_RECHARGE.getType())) {
                        throw exception(CREDIT_NOT_ENABLED);
                    }
                    // 授信额度 充值
                    if (distributorDO.getPayCredit() != null) {
                        beforeBalance = distributorDO.getSurplus();
                        afterBalance = distributorDO.getSurplus().subtract(money);
                        BigDecimal creditRemained = distributorDO.getPayCredit().subtract(afterBalance);
                        tradeName = "授信充值金额：" + money + "元,授信总额度" + distributorDO.getPayCredit() + "元,充值后剩余额度" + creditRemained + "元;已用额度" + afterBalance + "元";
                    } else {
                        tradeName = "授信充值金额：" + money + "元,授信总额度不限,充值后剩余额度不限;已用额度" + afterBalance + "元";
                    }
                    tradeType = DistributorTradeTypeEnum.CREDIT_RECHARGE.getType();
                    flag = distributorService.subSurplus(distributorDO.getId(), money);
                } else if (ObjectUtil.equals(rechargeDO.getRechargeType(), DistributorTradeTypeEnum.STOCK_DIRECT_RECHARGE.getType())) {
                    if (typeStream.noneMatch(element -> Integer.parseInt(element) == DistributorTradeTypeEnum.STOCK_DIRECT_RECHARGE.getType())) {
                        throw exception(CREDIT_NOT_INVENTORY);
                    }
                    beforeBalance = distributorDO.getInventory();
                    afterBalance = beforeBalance.add(money);
                    tradeName = "定向库存充值金额：" + money + "元,充值后余额为" + afterBalance + "元。";
                    tradeType = DistributorTradeTypeEnum.STOCK_DIRECT_RECHARGE.getType();
                    flag = distributorService.addInventory(distributorDO.getId(), money);
                }
                Long userId = getLoginUserId();
                if (flag) {
                    //添加交易记录数据
                    TradeLogSaveReqVO logVO = new TradeLogSaveReqVO();
                    logVO.setOperatorId(userId);
                    logVO.setRemark(tradeName);
                    logVO.setTradeMode(1);
                    logVO.setTradeType(tradeType);
                    // 余额表示交易前金额 授信表示交易前已使用额度
                    logVO.setBeforeBalance(beforeBalance);
                    // 余额表示交易后金额 授信表示交易后已使用额度
                    logVO.setAfterBalance(afterBalance);
                    logVO.setTradeAmount(money);
                    logVO.setTradeTime(LocalDateTime.now());
                    logVO.setDistributorId(rechargeDO.getDistributorId());

                    Long tradeId = tradeLogService.createTradeLog(logVO);
                    if (tradeId != null) {
                        return tradeId;
                    }
                }
            }
        } catch (ServiceException e) {
            log.error("[分销商充值]{}", e.getMessage());
            throw new ServiceException(e.getCode(), e.getMessage());
        } catch (Exception ex) {
            log.error("[分销商充值]", ex);
            throw exception(DISTRIBUTOR_RECHARGE_FAILED);
        } finally {
            if (lock.isLocked() && lock.isHeldByCurrentThread()) {
                lock.unlock();
            }
        }
        return null;
    }

    @Override
    @Transactional
    @TenantIgnore
    public CommonResult<DistributorRechargeRespDTO> tradeRecharge(DistributorRechargeReqDTO rechargeReqDTO) {
        RLock lock = null;
        DistributorRechargeRespDTO respDTO = null;
        try {
            //交易类型(1 余额充值、2 授信充值、3定向库存充值、4 余额购票、5余额退票、7授信购票、8授信退款、9定向库存购买、10定向库存退款、11预付款自助充值、12定向库存购买、13定向库存回收)
            int tradeType = rechargeReqDTO.getTradeType();
            // 支付方式
            int payType = rechargeReqDTO.getRechargeType();
            if (payType == 1
                    && (tradeType == DistributorTradeTypeEnum.BALANCE_STOCK_DIRECT_RECHARGE.getType()
                    || tradeType == DistributorTradeTypeEnum.BALANCE_STOCK_DIRECT_BACK.getType())) {
                Long userId = getLoginUserId();
                Long tradeId = tradeLogService.distributorRecharge(Long.parseLong(rechargeReqDTO.getAccountId()), Long.parseLong(rechargeReqDTO.getOrderId()), tradeType, userId, "", rechargeReqDTO.getAmount());
                respDTO = new DistributorRechargeRespDTO();
                respDTO.setRechargeType(tradeType);
                respDTO.setTradeId(tradeId);
                return CommonResult.success(respDTO);
            }
            String key = "ota_" + rechargeReqDTO.getAccountId();
            if (payType == DistributorTradeTypeEnum.BALANCE_RECHARGE.getType()) {
                //预付款
                key += "_balance";
            } else if (payType == DistributorTradeTypeEnum.CREDIT_RECHARGE.getType()) {
                //授信
                key += "_credit";
            } else {
                //定向库存
                key += "_inventory";
            }
            lock = redissonClient.getLock(key);
            if (lock.tryLock(10, 30, TimeUnit.SECONDS)) {
                DistributorDO distributor = distributorService.getDistributor(Long.parseLong(rechargeReqDTO.getAccountId()));
                Asserts.isTrue(null != distributor, "分销商{}不存在", rechargeReqDTO.getAccountId());
                //判断是否自营分销商 若不是自营分销商则需扣除
                if (distributor.getSelfSupport() == 0 || ObjectUtil.equals(distributor.getType(), DistributorTypeEnum.OTC.getType())) {
                    //交易额
                    BigDecimal money = rechargeReqDTO.getAmount();
                    boolean flag = false;
                    String tradeName = "";
                    //交易前金额
                    BigDecimal beforeBalance = BigDecimal.ZERO;
                    //交易后金额
                    BigDecimal afterBalance = BigDecimal.ZERO;
                    int tradeMode = 1;
                    int originTradeType = tradeType;
                    Supplier<Stream<String>> streamSupplier = () -> Stream.of(distributor.getPayType().split(","));
                    boolean isUseBalance = streamSupplier.get().anyMatch(element -> Integer.parseInt(element) == DistributorTradeTypeEnum.BALANCE_RECHARGE.getType());
                    boolean isUseCredit = streamSupplier.get().anyMatch(element -> Integer.parseInt(element) == DistributorTradeTypeEnum.CREDIT_RECHARGE.getType());
                    //消费
                    switch (tradeType) {
                        case 4:

                            //交易前金额
                            beforeBalance = distributor.getBalance();
                            afterBalance = beforeBalance.subtract(money);
                            //是否可以切换到授信支付
                            tradeMode = 2;
                            boolean canChangeToCredit = true;
                            if (rechargeReqDTO.getIsFixed() == 1 || streamSupplier.get().noneMatch(element -> ObjectUtil.equals(element, DistributorTradeTypeEnum.CREDIT_RECHARGE.getType().toString()))) {
                                canChangeToCredit =false;
                            }

                            if(tradeType == DistributorTradeTypeEnum.CREDIT_TRANS.getType()){

                                rechargeReqDTO.setRechargeType(DistributorTradeTypeEnum.CREDIT_RECHARGE.getType());
                                rechargeReqDTO.setTradeType(tradeType);
                                return tradeRecharge(rechargeReqDTO);
                            }

                            break;
                        case 5:
                        case 10:
                            //预付款退款 定向库存退款
                            beforeBalance = distributor.getBalance();
                            //交易后金额
                            afterBalance = beforeBalance.add(money);
                            AtomicBoolean isShareBalance = new AtomicBoolean(false);



                            break;
                        case 2:
                            //授信账单还款充值
                            break;
                        case 8:
                            //授信退款
                            beforeBalance = distributor.getSurplus();
                            //退款后已用额度 = 当前已用额度 - 退款金额
                            afterBalance = beforeBalance.subtract(money);

                            break;
                        case 9:
                            //交易前金额
                            beforeBalance = distributor.getBalance();
                            afterBalance = beforeBalance.subtract(money);

                            break;
                        case 14:
                            //营销政策充值
                            beforeBalance = distributor.getBalance();
                            afterBalance = beforeBalance.add(money);
                            tradeName = "营销政策充值金额：" + money + "元,充值到账后剩余余额" + afterBalance + "元";
                            flag = distributorService.addBalance(distributor.getId(), money);
                            break;
                        case 15:
                            //营销政策撤回
                            beforeBalance = distributor.getBalance();
                            afterBalance = beforeBalance.subtract(money);
                            tradeName = "营销政策撤回金额：" + money + "元,营销政策撤回后剩余余额" + afterBalance + "元";
                            flag = distributorService.subBalance(distributor.getId(), money);
                            break;
                        case 16:
                            //预付款支付授信账单
                            beforeBalance = distributor.getBalance();
                            afterBalance = beforeBalance.subtract(money);
                            tradeMode = 2;
                            isShareBalance = new AtomicBoolean(false);

                            break;
                        case 17:
                            //给门店分配预付款
                            Asserts.isTrue(rechargeReqDTO.getStoreId()!=null,"门店预付款充值，门店ID必传");
                            tradeMode = rechargeReqDTO.getMode();
                            //预付款
                            beforeBalance = distributor.getBalance();

                            break;
                        case 18:
                            Asserts.isTrue(rechargeReqDTO.getStoreId()!=null,"门店授信分配，门店ID必传");
                            tradeMode = rechargeReqDTO.getMode();
                            //平台给门店分配授信
                            beforeBalance = distributor.getPayCredit();
                            break;
                        default:
                            break;
                    }
                    // 授信付款
                    if (tradeType == DistributorTradeTypeEnum.CREDIT_TRANS.getType()) {

                        tradeMode = 2;
                    }

                    respDTO = new DistributorRechargeRespDTO();
                    respDTO.setRechargeType(tradeType);
                    if (flag) {

                        //添加ota消费记录数据
                        TradeLogDO logDO = new TradeLogDO();
                        logDO.setOperatorId(distributor.getId());
                        logDO.setOperatorName(distributor.getName());
                        logDO.setRemark(tradeName);
                        logDO.setTradeMode(tradeMode);
                        logDO.setTradeType(tradeType);
                        // 余额表示交易前金额 授信表示交易前已使用额度
                        logDO.setBeforeBalance(beforeBalance);
                        // 余额表示交易后金额 授信表示交易后已使用额度
                        logDO.setAfterBalance(afterBalance);
                        logDO.setTradeAmount(money);
                        logDO.setTradeTime(LocalDateTime.now());
                        logDO.setDistributorId(distributor.getId());
                        logDO.setTenantId(distributor.getTenantId());
                        logDO.setOrderId(rechargeReqDTO.getOrderId());
                        logDO.setStoreId(rechargeReqDTO.getStoreId());
                        Long tradeId = tradeLogService.saveTradeLog(logDO);
                        respDTO.setTradeId(tradeId);
                        respDTO.setRechargeType(tradeType);
                    } else {
                        if(!(tradeType == DistributorTradeTypeEnum.CREDIT_RECHARGE.getType() && rechargeReqDTO.getStoreId()!=null)){
                            throw exception(DISTRIBUTOR_PAY_FAILED);
                        }
                    }
                }
            }
        } catch (ServiceException e) {
            log.error("[分销商支付]{}", e);
            throw new ServiceException(e.getCode(), e.getMessage());
        } catch (Exception e) {
            log.error("[分销商支付]", e);
            throw exception(DISTRIBUTOR_PAY_FAILED);
        } finally {
            if (null != lock && lock.isLocked() && lock.isHeldByCurrentThread()) {
                lock.unlock();
            }
        }
        return CommonResult.success(respDTO);
    }

}
