package com.yc.ship.module.resource.controller.admin.port;

import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import com.yc.ship.framework.common.pojo.PageParam;
import com.yc.ship.framework.common.pojo.PageResult;
import com.yc.ship.framework.common.pojo.CommonResult;
import com.yc.ship.framework.common.util.object.BeanUtils;
import static com.yc.ship.framework.common.pojo.CommonResult.success;

import com.yc.ship.framework.excel.core.util.ExcelUtils;

import com.yc.ship.framework.apilog.core.annotation.ApiAccessLog;
import static com.yc.ship.framework.apilog.core.enums.OperateTypeEnum.*;

import com.yc.ship.module.resource.controller.admin.port.vo.*;
import com.yc.ship.module.resource.dal.dataobject.port.ResourcePortDO;
import com.yc.ship.module.resource.service.port.ResourcePortService;

@Tag(name = "管理后台 - 资源管理-口岸管理")
@RestController
@RequestMapping("/resource/port")
@Validated
public class ResourcePortController {

    @Resource
    private ResourcePortService portService;

    @PostMapping("/create")
    @Operation(summary = "创建资源管理-口岸管理")
    @PreAuthorize("@ss.hasPermission('resource:port:create')")
    public CommonResult<Long> createPort(@Valid @RequestBody ResourcePortSaveReqVO createReqVO) {
        return success(portService.createPort(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新资源管理-口岸管理")
    @PreAuthorize("@ss.hasPermission('resource:port:update')")
    public CommonResult<Boolean> updatePort(@Valid @RequestBody ResourcePortSaveReqVO updateReqVO) {
        portService.updatePort(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除资源管理-口岸管理")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('resource:port:delete')")
    public CommonResult<Boolean> deletePort(@RequestParam("id") Long id) {
        portService.deletePort(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得资源管理-口岸管理")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('resource:port:query')")
    public CommonResult<ResourcePortRespVO> getPort(@RequestParam("id") Long id) {
        ResourcePortDO port = portService.getPort(id);
        return success(BeanUtils.toBean(port, ResourcePortRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得资源管理-口岸管理分页")
    @PreAuthorize("@ss.hasPermission('resource:port:query')")
    public CommonResult<PageResult<ResourcePortRespVO>> getPortPage(@Valid ResourcePortPageReqVO pageReqVO) {
        PageResult<ResourcePortDO> pageResult = portService.getPortPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, ResourcePortRespVO.class));
    }

    @GetMapping("/enableList")
    @Operation(summary = "获得启用的口岸列表")
    public CommonResult<List<ResourcePortRespVO>> enableList() {
        List<ResourcePortDO> list = portService.enableList();
        return success(BeanUtils.toBean(list, ResourcePortRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出资源管理-口岸管理 Excel")
    @PreAuthorize("@ss.hasPermission('resource:port:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportPortExcel(@Valid ResourcePortPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<ResourcePortDO> list = portService.getPortPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "资源管理-口岸管理.xls", "数据", ResourcePortRespVO.class,
                        BeanUtils.toBean(list, ResourcePortRespVO.class));
    }

}